﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace WindowsFormsApp1
{
    //Třída UI se stará o uživatelské rozhraní
    class UI
    {
        //Proměnné uživatelského rozhraní, slovník chybových hlášek, řídících barev, font, vzhed a logickou mapu UI
        Dictionary<string, string> errorMessages = new Dictionary<string, string>
        {
            {"Missing map","The associated map file was not found in application directory" },
            {"Incorrect map","Map does not have the correct format" },
            {"Duplicate keys","Key colors in Map file have to be distinct" },
            {"Duplicate textures","Texture keys in Map file have to be distint" },
            {"No spawnpoint","Spawnpoint has to be specified on the first map" },
            {"Incomplete borders","All maps need to have a solid outer square border" },
        };
        Color small, medium, large, ok, exit, start, resume, editor, replay, save;
        Dictionary<Color, string> buttons;
        Dictionary<char, int> compactFont = new Dictionary<char, int>
        {
            {'A', 27129},
            {'B', 61343},
            {'C', 30855},
            {'D', 59806},
            {'E', 65167},
            {'F', 63720},
            {'G', 30871},
            {'H', 39417},
            {'I', 61999},
            {'J', 61847},
            {'K', 39401},
            {'L', 34959},
            {'M', 40857},
            {'N', 40377},
            {'O', 27030},
            {'P', 59880},
            {'Q', 27062},
            {'R', 59881},
            {'S', 31806},
            {'T', 62532},
            {'U', 39327},
            {'V', 39318},
            {'W', 39359},
            {'X', 39273},
            {'Y', 39268},
            {'Z', 62031},
            {'0', 63903},
            {'1', 27183},
            {'2', 57711},
            {'3', 63263},
            {'4', 35570},
            {'5', 64575},
            {'6', 30975},
            {'7', 61730},
            {'8', 65439},
            {'9', 63985},
            {'+', 28662},
            {'↑', 1776},
            {'↓', 3936},
            {' ', 0 }
        };
        Bitmap logicMap, cosmeticMap;
        Size uiSize;

        //Při konstrukci se UI přispůsobí velikosti herního panelu
        public UI(Size size)
        {
            Resize(size);
            //Určí se řídící barvy, a přiřadí se jim funkce, takto lze zajistit že bude možné barvy ve slovníku vyhledávat
            buttons = new Dictionary<Color, string>
            {
                {small = Color.FromArgb(200,200,200), "500 x 250" },
                {medium = Color.FromArgb(150,150,150), "750 x 375" },
                {large = Color.FromArgb(100,100,100), "1000 x 500" },
                {ok = Color.FromArgb(50,50,50), "Ok" },
                {exit = Color.FromArgb(255,0,0), "Exit" },
                {start = Color.FromArgb(0,255,0), "Start" },
                {resume = Color.FromArgb(0,0,255), "Resume" },
                {editor = Color.FromArgb(255,255,0), "Editor" },
                {replay = Color.FromArgb(0,255,255), "Replay" },
                {save = Color.FromArgb(255,0,255), "Save" },
            };
        }

        //Metoda Rezize změní všechny hodnoty řídící velikost UI na novou velikost
        public void Resize(Size size)
        {
            uiSize = size;
            logicMap = new Bitmap(uiSize.Width, uiSize.Height);
            cosmeticMap = new Bitmap(uiSize.Width, uiSize.Height);
        }

        //Metoda WeHappy je zavolána po načtení souborů
        public bool WeHappy(string errorCode)
        {
            //Pokud neobdrží žádný kód chyby, vrátí true a umožní spuštění hry
            if (errorCode == "") return true;
            //Pokud obdrží kód chyby, vyhodí messagebox s jejím popisem, po jeho zavření vrátí false, čímž aplikaci zavře
            else if (errorMessages.ContainsKey(errorCode)) MessageBox.Show(errorMessages[errorCode], errorCode, MessageBoxButtons.OK);
            else MessageBox.Show(errorCode, "Unknown error", MessageBoxButtons.OK);
            return false;
        }

        //Metoda WriteText vytvoří bitmapu se specifikovaným nápisem
        protected Bitmap WriteText(string text)
        {
            //Výška bitmapy je 4 pixely, a šířka je 4 pixely, krát počet znaků, plus jeden pixel mezi každými dvěma znaky
            Bitmap bmp = new Bitmap(4 * text.Length + text.Length - 1, 4);
            //Text je převeden na velké znaky, protože výpis malé neobsahuje, a nepodporuje
            text = text.ToUpper();
            using (Graphics g = Graphics.FromImage(bmp))
            {
                g.Clear(Color.Black);
                //Za každý znak se buď zavolá metoda DrawLetter pokud se jedná o platný znak, nebo se vykreslí čtverec
                for (int i = 0; i < text.Length; i++)
                {
                    if (compactFont.ContainsKey(text[i])) g.DrawImage(DrawLetter(text[i]), new Point(i * 5, 0));
                    else g.FillRectangle(Brushes.White, i * 5, 0, 4, 4);
                }
            }
            //Černá barva je zprůhledněna, protože to při pauze hry vypadá lépe
            bmp.MakeTransparent(Color.Black);
            return bmp;
        }

        //Metoda DrawLetter vykresluje písmeno ze slovníku compactFont
        private Bitmap DrawLetter(Char letter)
        {
            //CompactFont se skládá ze znaků reprezentovaných jen šestnácti pixely, proto je každé písmeno bitmapa 4x4
            Bitmap bmp = new Bitmap(4, 4);
            //Podle Char klíče se ve slovníku najde int hodnota, která se převede na šestnáctibitové binární číslo
            string bin = Convert.ToString(compactFont[letter], 2).PadLeft(16, '0');
            //Každý z pixelů bitmapy je postupně podle odpovídající binární hodnoty v pořadí vybarven buď bíle pro 1, nebo černě pro 0
            for (int y = 0; y < bmp.Height; y++)
            {
                for (int x = 0; x < bmp.Width; x++)
                {
                    bmp.SetPixel(x, y, bin[x + 4 * y] == '1' ? Color.White : Color.Black);
                }
            }
            return bmp;
        }

        //UI se skládá z vzhledové a logické mapy. Vzhledová mapa se vykresluje a neplní žádnou logickou funkci, logická mapa zůstává skrytá, a
        //rozeznává kliknutí. Oblasti kde jsou na vzhledové mapě nápisy tlačítek jsou na logické mapě vyplněny odpovídající barvou. Při kliknutí
        //se zjistí barva odpovídajícího pixelu na logické mapě, a podle jeho barvy se odešle funkce. Takto je možné se vyhnout používání tlačítek
        //nebo jiných klikatelných formulářových prvků, nebo porovnávání souřadnic kliknutí s vybranými regiony. Navíc je možné snadno vytvořit
        //klikatelnou oblast jakéhokoli tvaru
        public string UIClick(Point mouse)
        {
            Color c = logicMap.GetPixel(mouse.X, mouse.Y);
            if (buttons.ContainsKey(c)) return buttons[c];
            else return "";
        }

        //Následující metody vytvářejí grafické a logické mapy UI. Každé menu má vlastní metodu. Grafická mapa je předána k vykreslení na herní
        //panel, logická je upravena, ale zůstává pouze uvnitř objektu. Každá metoda cyklicky vykresluje nápisy
        //odpovídající zadaným klíčům ze slovníku buttons. Jejich velikost a umístění je parametricky závislé na velikosti herího panelu.
        //V některých případech je navíc vykreslován neaktivní nadpis, nebo hodnota skore. Nápisy jsou zvětšovány vykreslením do prostoru většího
        //než samy jsou, proto je u jejich vykreslování vypnuté vyhlazování. Metody by bylo možné lépe standardizovat, ale tím by se zhoršila
        //přístupnost pro úpravy individuálních menu, a jednoduchost jejich volání. 

        public Bitmap SizeSelectMenu()
        {
            Color[] thisButtons = new Color[] { small, medium, large, ok };
            int rectSide = uiSize.Height / (thisButtons.Length * 2 + 1);
            using (Graphics g = Graphics.FromImage(logicMap))
            {
                g.Clear(Color.Transparent);
                for (int i = 0; i < thisButtons.Length; i++)
                {
                    g.FillRectangle(new SolidBrush(thisButtons[i]), new Rectangle(uiSize.Width / 2 - (rectSide * buttons[thisButtons[i]].Length / 2), (1 + i * 2) * rectSide, rectSide * buttons[thisButtons[i]].Length, rectSide));
                }
            }
            using (Graphics g = Graphics.FromImage(cosmeticMap))
            {
                g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
                g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.Half;
                g.Clear(Color.Transparent);
                for (int i = 0; i < thisButtons.Length; i++)
                {
                    g.DrawImage(WriteText(buttons[thisButtons[i]]), new Rectangle(uiSize.Width / 2 - (rectSide * buttons[thisButtons[i]].Length / 2), (1 + i * 2) * rectSide, rectSide * buttons[thisButtons[i]].Length, rectSide));
                }
            }
            return cosmeticMap;
        }

        public Bitmap MainMenu()
        {
            Color[] thisButtons = new Color[] { start, editor, replay, exit };
            int rectSide = uiSize.Height / (thisButtons.Length * 2 + 5);
            using (Graphics g = Graphics.FromImage(logicMap))
            {
                g.Clear(Color.Transparent);
                for (int i = 0; i < thisButtons.Length; i++)
                {
                    g.FillRectangle(new SolidBrush(thisButtons[i]), new Rectangle(uiSize.Width / 2 - (rectSide * buttons[thisButtons[i]].Length / 2), (5 + i * 2) * rectSide, rectSide * buttons[thisButtons[i]].Length, rectSide));
                }
            }
            using (Graphics g = Graphics.FromImage(cosmeticMap))
            {
                g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
                g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.Half;
                g.Clear(Color.Transparent);
                string title = "SPOOKY";
                g.DrawImage(WriteText(title), new Rectangle(uiSize.Width / 2 - (rectSide * title.Length), (int)(1.5f * rectSide), rectSide * title.Length * 2, rectSide * 2));
                for (int i = 0; i < thisButtons.Length; i++)
                {
                    g.DrawImage(WriteText(buttons[thisButtons[i]]), new Rectangle(uiSize.Width / 2 - (rectSide * buttons[thisButtons[i]].Length / 2), (5 + i * 2) * rectSide, rectSide * buttons[thisButtons[i]].Length, rectSide));
                }
            }
            return cosmeticMap;
        }

        public Bitmap PauseMenu()
        {
            Color[] thisButtons = new Color[] { resume, exit };
            int rectSide = uiSize.Height / (thisButtons.Length * 2 + 1);
            using (Graphics g = Graphics.FromImage(logicMap))
            {
                g.Clear(Color.Transparent);
                for (int i = 0; i < thisButtons.Length; i++)
                {
                    g.FillRectangle(new SolidBrush(thisButtons[i]), new Rectangle(uiSize.Width / 2 - (rectSide * buttons[thisButtons[i]].Length / 2), (1 + i * 2) * rectSide, rectSide * buttons[thisButtons[i]].Length, rectSide));
                }
            }
            using (Graphics g = Graphics.FromImage(cosmeticMap))
            {
                g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
                g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.Half;
                g.Clear(Color.Transparent);
                for (int i = 0; i < thisButtons.Length; i++)
                {
                    g.DrawImage(WriteText(buttons[thisButtons[i]]), new Rectangle(uiSize.Width / 2 - (rectSide * buttons[thisButtons[i]].Length / 2), (1 + i * 2) * rectSide, rectSide * buttons[thisButtons[i]].Length, rectSide));
                }
            }
            return cosmeticMap;
        }

        public Bitmap WinMenu(int seconds)
        {
            Color[] thisButtons = new Color[] { ok, save };
            int rectSide = uiSize.Height / (thisButtons.Length * 2 + 5);
            using (Graphics g = Graphics.FromImage(logicMap))
            {
                g.Clear(Color.Transparent);
                for (int i = 0; i < thisButtons.Length; i++)
                {
                    g.FillRectangle(new SolidBrush(thisButtons[i]), new Rectangle(uiSize.Width / 2 - (rectSide * buttons[thisButtons[i]].Length / 2), (5 + i * 2) * rectSide, rectSide * buttons[thisButtons[i]].Length, rectSide));
                }
            }
            using (Graphics g = Graphics.FromImage(cosmeticMap))
            {
                g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
                g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.Half;
                g.Clear(Color.Transparent);
                string title = "You win";
                string subtitle = seconds + " s";
                g.DrawImage(WriteText(title), new Rectangle(uiSize.Width / 2 - (rectSide * title.Length / 2), rectSide, rectSide * title.Length, rectSide));
                g.DrawImage(WriteText(subtitle), new Rectangle(uiSize.Width / 2 - (rectSide * subtitle.Length / 2), 3 * rectSide, rectSide * subtitle.Length, rectSide));
                for (int i = 0; i < thisButtons.Length; i++)
                {
                    g.DrawImage(WriteText(buttons[thisButtons[i]]), new Rectangle(uiSize.Width / 2 - (rectSide * buttons[thisButtons[i]].Length / 2), (5 + i * 2) * rectSide, rectSide * buttons[thisButtons[i]].Length, rectSide));
                }
            }
            return cosmeticMap;
        }
    }
}
